//---------------------------------------------------------------------------
#ifndef TJabberH
#define TJabberH
//---------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
//---------------------------------------------------------------------------
class TJabber : public TComponent
{
private:
	// Events
	typedef void __fastcall (__closure *TNewNoteEvent)(TObject *Sender, AnsiString From, AnsiString FromNickName, AnsiString Ext, AnsiString Thread, AnsiString Priority, AnsiString Subject, AnsiString Message);
	TNewNoteEvent FOnNewNote;

	typedef void __fastcall (__closure *TServerInfoEvent)(TObject *Sender, AnsiString Version, AnsiString Protocol);
	TServerInfoEvent FOnServerInfo;
	
	typedef void __fastcall (__closure *TSendDataEvent)(TObject *Sender, AnsiString Data);
	TSendDataEvent FOnSendData;

	typedef void __fastcall (__closure *TRosterUpdateEvent)(TObject *Sender);
	TRosterUpdateEvent FOnRosterUpdate;



	// Structs
	struct attribute
	{
	    	attribute() { Next = NULL; };
	        ~attribute() { delete Next; };
		AnsiString Name;
		AnsiString Value;
		attribute *Next;
	};
	struct tag
	{
		AnsiString Name;
		attribute* Attribs;
		AnsiString Contents;
	};
	struct user
	{
	    	user() { Next = NULL; };
	        ~user() { delete Next; };
		AnsiString UserName;
		AnsiString NickName;
		AnsiString StatusLine;
		AnsiString StatusType;
		AnsiString Status;
		user *Next;
	};
	struct roster
	{
		roster() { Next = NULL; Group=""; };
		~roster() { delete Next; };
		AnsiString Group;
		AnsiString UserName;
		roster *Next;
	};
	struct group
	{
		group() { Next=NULL; Group=""; };
		~group() { delete Next; };
		AnsiString Group;
		group *Next;
	};


	
	// Internal support functions
	AnsiString __fastcall ParseXML(tag* Tag, bool& Whole, AnsiString XML);

	AnsiString __fastcall AttrName(attribute* Attrib, AnsiString Name);   // Finds Name in Attrib and returns Value
	AnsiString __fastcall AttrValue(attribute* Attrib, AnsiString Value); // Opposite of AttrName()

	void __fastcall GroupAdd(group* G, AnsiString Group);  // Adds Group to group G.  Ignores if duplicate.
	void __fastcall RosterAdd(roster* Roster, AnsiString Group, AnsiString UserName);  // Adds UserName and Group to Roster.  Ignores if duplicate.

	void __fastcall FetchRosterGroups(void);	     // Get the rosters for each known group

	void __fastcall HandleCommand(tag* Tag);
	AnsiString __fastcall DecodeString(AnsiString String);

	void __fastcall SendData(void);

	void __fastcall SetPClientStatus(AnsiString Status);
	void __fastcall SetPClientPriority(int Pri);


	// Regular vars
	AnsiString PROTOCOL;
	AnsiString InputBuffer;
	AnsiString OutputBuffer;
	bool Connected;
	bool CauseRosterEvent;

	AnsiString PClientName;
	int PClientPriority;
	AnsiString PClientStatus;

	AnsiString PUserName;
	AnsiString PUserNickName;
	AnsiString PUserPassword;

	roster *MainRoster;
	group *GroupList;

protected:
public:
	__fastcall TJabber(TComponent* Owner);
	void __fastcall LoadData(AnsiString Input);  // Loads Input into Jabber buffer and processes the data calling events when needed
	int __fastcall Connect(void);		     // Uses data in properties and then calls OnSendData to send inital connection status packets and sets Connect var to true.  It returns 1 if happy and 0 if not.
	int __fastcall Disconnect(void);	     // Sends disconnect packet to OnSendData and reset the Connected var to false.  It returns 1 if happy and 0 if not.
	void __fastcall FetchRoster(void);	     // Starts the process of getting an updated list of the rosters from the server.  It will eventually cause an OnRosterUpdate event if there is anything listed in the roster.
	void __fastcall RosterGroups(TStringList* List); // Sends back a TStringList with the list of groups the server knows about.
	void __fastcall RosterList(AnsiString Group,TStringList* List); // Sends back a TStringList with the list of users in Group.
	void __fastcall RosterAddUser(AnsiString Group, AnsiString UserName); // Add UserName to Group roster
	void __fastcall SendJabberNote(AnsiString To, AnsiString ToNickName, AnsiString Ext, AnsiString Thread, AnsiString Priority, AnsiString Subject, AnsiString Message); //Sends a new JabberNote to someone

__published:
	// Properties
	__property AnsiString UserName={read=PUserName, write=PUserName};
	__property AnsiString UserNickName={read=PUserNickName, write=PUserNickName};
	__property AnsiString UserPassword={read=PUserPassword, write=PUserPassword};

	__property AnsiString ClientName={read=PClientName, write=PClientName};
	__property int ClientPriority={read=PClientPriority, write=SetPClientPriority};
	__property AnsiString ClientStatus={read=PClientStatus, write=SetPClientStatus};


	// Events
	__property TNewNoteEvent OnNewNote={read=FOnNewNote, write=FOnNewNote};
	__property TServerInfoEvent OnServerInfo={read=FOnServerInfo, write=FOnServerInfo};
	__property TSendDataEvent OnSendData={read=FOnSendData, write=FOnSendData};
	__property TRosterUpdateEvent OnRosterUpdate={read=FOnRosterUpdate, write=FOnRosterUpdate};
};
//---------------------------------------------------------------------------
#endif
